//===============================================================================
// Microsoft Services
// Microsoft.Services.TfsAlert.Core
// TFSAlerts
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
using System;
using System.Configuration;
using System.Collections.Generic;
using System.Diagnostics;
using Microsoft.TeamFoundation.Server;

using Microsoft.Services.TfsAlert.AppCode;
using Microsoft.Services.TfsAlert.Core;
using Microsoft.Services.TfsAlert.Core.BusinessFacades.RegistrationBF;
using Microsoft.Services.TfsAlert.Core.BusinessFacades.TeamFoundationBF;
using Microsoft.Services.TfsAlert.Core.Model;



namespace Microsoft.Services.TfsAlert.Core.BusinessFacades.RegistrationBF.Impl
{
    internal partial class RegistrationManagerImpl : IRegistrationManager
    {

        #region Private Instance Members
        private ApplicationHelpers _applicationHelpers;
        private TeamFoundationFactory _teamFoundationFactory = null;
        private IBusinessFacadeFactory _businessFacadeFactory = null;
        ITeamFoundationFacade _foundationFacade = null;
        private List<TfsServer> _teamFoundationServers = null;
        #endregion

        #region Constructors
        //TODO REFACTOR TO internal and fix unit tests
        public RegistrationManagerImpl()
        {
            Initilize();
        }
        #endregion

        #region Properties
        private List<TfsServer> TeamFoundationServers
        {
            get
            {
                if (_teamFoundationServers == null)
                {
                    try
                    {
                        this._teamFoundationServers = _foundationFacade.RetrieveAllServers();
                    }
                    catch (Exception _ex)
                    {

                        Trace.TraceError("Unable To Retrieve Servers");
                        Trace.TraceError(_ex.Message);
                        Trace.WriteLine(_ex);
                        throw;
                        //cant retrieve servers  we should probably log.
                    }
                }
                return this._teamFoundationServers;
            }
            set
            {
                _teamFoundationServers = value;
            }
        }

        private TeamFoundationFactory teamFoundationFactory
        {
            get
            {
                if (null == _teamFoundationFactory)
                {
                    _teamFoundationFactory = new TeamFoundationFactory();
                    return _teamFoundationFactory;
                }
                return _teamFoundationFactory;
            }

            set { _teamFoundationFactory = value; }
        }

        private IEventService _tfsEventService = null;

        private IEventService tfsEventService
        {
            get
            {
                if (null == _tfsEventService)
                {
                    foreach (TfsServer _server in this.TeamFoundationServers)
                    {
                        _tfsEventService = teamFoundationFactory.GetEventService(_server.Name);
                    }
                    // _tfsEventService = teamFoundationFactory.GetEventService( tfsEventCollection.TeamFoundationServer );
                    return _tfsEventService;
                }
                return _tfsEventService;
            }

            set { _tfsEventService = value; }
        }

        #endregion //Properties

        #region IRegistrationManager Members

        public void Suscribe()
        {
            //Failsafe cleanup.
            this.UnSuscribe();

            foreach (TfsServer _server in this.TeamFoundationServers)
            {
                foreach (TfsEvent _event in _server.Events)
                {
                    if (_event.IsSelected)
                    {
                        int _subscriptionID;

                        _subscriptionID = this.tfsEventService.SubscribeEvent(
                                                _server.UserName,
                                                _event.EventType,
                                                _event.FilterExpression,
                                                this.getDeliveryPreference());

                        _event.SubscriptionID = _subscriptionID;
                    }
                }
            }
            //TODO Why are we updating here if nothing has changed why update?
            this._foundationFacade.Update(this.TeamFoundationServers);        
        }

        public void UnSuscribe()
        {
            foreach (TfsServer _server in this.TeamFoundationServers)
            {
                foreach (TfsEvent _event in _server.Events)
                {
                    if (_event.IsSelected && _event.SubscriptionID > 0)
                    {
                        this.tfsEventService.UnsubscribeEvent(_event.SubscriptionID);

                        _event.SubscriptionID = 0;
                    }
                }
            }
            //TODO Why are we updating here if nothing has changed why update?
            _foundationFacade.Update(this.TeamFoundationServers);
         
        }

        #endregion

        #region Private Members
        private void Initilize()
        {
            _applicationHelpers = new ApplicationHelpers();
            _businessFacadeFactory = BusinessFacadeFactory.CreateInstance();
            _foundationFacade = _businessFacadeFactory.CreateTeamFoundationBF();
        }

        private DeliveryPreference getDeliveryPreference()
        {
            DeliveryPreference deliveryPreferemce = new DeliveryPreference();
            deliveryPreferemce.Address = _applicationHelpers.BuildFullAddress().ToString();
            deliveryPreferemce.Schedule = DeliverySchedule.Immediate;
            deliveryPreferemce.Type = DeliveryType.Soap;

            return deliveryPreferemce;
        }
        #endregion

    }
}
