//===============================================================================
// Microsoft Services
// Microsoft.Services.TfsAlert.Core
// TFSAlerts
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
using System;
using System.Collections.Generic;
using System.IO;
using System.Security;
using System.Text;
using System.Xml;

using Microsoft.Services.TfsAlert.Core;
using Microsoft.Services.TfsAlert.Core.Model;
//using Microsoft.Services.TfsAlert.Core.DataAccessLayer.Model;


namespace Microsoft.Services.TfsAlert.Core.DataAccessLayer.TeamFoundationDL.Impl
{
    internal class TeamFoundationDLImpl : ITeamFoundationDataFacade
    {
        #region Contructors
        internal TeamFoundationDLImpl()
        {
        }
        #endregion

        #region ITeamFoundationDataFacade Members

        public List<TfsServer> RetrieveAllServers()
        {
            ApplicationHelpers _helper = new ApplicationHelpers();
            List<TfsServer> _servers = new List<TfsServer>();
            
            //check if file exits
            if (!File.Exists(_helper.GetTFSDataFilePath()))
            {
                //if the file doesnt exists return a empty list. 
                return _servers;
               
            }
            //deserialize the data if it exists
            XmlDocument _tfsAlertsXml = new XmlDocument();
            try
            {
                //lets load the xml file which contains a serialized version of TfsServer
                _tfsAlertsXml.Load(_helper.GetTFSDataFilePath());
                _servers = new List<TfsServer>(SerializationManager.XmlDeserialize<TfsServer[]>(_tfsAlertsXml));
            }
            catch (XmlException _ex) 
            {
                //lets log here
                System.Console.WriteLine(_ex);
                throw;
            }
           
            return _servers;       
        }

        public TfsServer RetrieveServer(string name)
        {
            throw new NotImplementedException("The method or operation is not implemented.");
        }

        public void Update(List<TfsServer> servers)
        {
            if (servers.Count.Equals(0))
            {
                throw new ArgumentException("Servers length value is 0.");
            }

            ApplicationHelpers _helper = new ApplicationHelpers();
            XmlDocument _doc = new XmlDocument();
            TfsServer[]  _servers;
            
            //Check the directory to see if it exists catch a SecurityException
            DirectoryInfo _info;
            try
            {
                _info = new DirectoryInfo(_helper.GetTFSProfilePath());
                if (!_info.Exists)
                {
                    _info.Create();
                }

            }
            catch (SecurityException _ex)
            {   
                //have to refactor this
                System.Console.WriteLine(_ex);
                throw;
            }
          
          
            //create TfsServers Array
             _servers = new TfsServer[servers.Count];
             _servers = servers.ToArray(); //convert to an object array
            
            // lets try to serialize to save the local profile
            try
            {
                _doc.LoadXml(SerializationManager.XmlSerialize<TfsServer[]>(_servers));
                _doc.Save(_helper.GetTFSDataFilePath());
            }
            catch (XmlException _ex)
            {
                //lets log
                System.Console.WriteLine(_ex);
                throw;
            }
        }

        public void Add(TfsServer server)
        {
            //retrieve the data and add this should probably be refactored
            List<TfsServer> _servers = this.RetrieveAllServers();
            _servers.Add(server);

            //calling Update
            this.Update(_servers);
        }

        #endregion

       
    }
}
