//===============================================================================
// Microsoft Services
// Microsoft.Services.TfsAlert.Core
// TFSAlerts
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Services.TfsAlert.Core.Model;
using Microsoft.Services.TfsAlert.Core.BusinessFacades.TeamFoundationBF;
using Microsoft.Services.TfsAlert.Core.DataAccessLayer.TeamFoundationDL;


namespace Microsoft.Services.TfsAlert.Core.BusinessFacades.TeamFoundationBF.Impl
{
    internal class TeamFoundationImpl : ITeamFoundationFacade
    {
        #region Constructors
        internal TeamFoundationImpl()
        {

        }
        #endregion

        #region ITeamFoundationFacade Members

        public List<TfsServer> RetrieveAllServers()
        {
            TeamFoundationDAFactory _factory = TeamFoundationDAFactory.CreateInstance();
            ITeamFoundationDataFacade _daFacade = _factory.CreateTeamFoundationDAFacade();
            return _daFacade.RetrieveAllServers();
         
        }

        public TfsServer RetrieveServer(string name)
        {
            if (string.IsNullOrEmpty(name))
            {
                throw new ArgumentException("name");
            }
            TeamFoundationDAFactory _factory = TeamFoundationDAFactory.CreateInstance();
            ITeamFoundationDataFacade _daFacade = _factory.CreateTeamFoundationDAFacade();
            
            return _daFacade.RetrieveServer(name);
        }

        public void Update(List<TfsServer> servers)
        {
            if (servers == null)
            {
                throw new ArgumentNullException("Servers");
            }
            TeamFoundationDAFactory _factory = TeamFoundationDAFactory.CreateInstance();
            ITeamFoundationDataFacade _daFacade = _factory.CreateTeamFoundationDAFacade();
            _daFacade.Update(servers);


        }

        public void Add(TfsServer server)
        {  
            TeamFoundationDAFactory _factory = TeamFoundationDAFactory.CreateInstance();
            ITeamFoundationDataFacade _daFacade = _factory.CreateTeamFoundationDAFacade();
            _daFacade.Add(server);
        }

     
        #endregion
    }
}
