//===============================================================================
// Microsoft Services
// Microsoft.Services.TfsAlert.Core
// TFSAlerts
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Xml;
using System.Xml.Serialization;


namespace Microsoft.Services.TfsAlert.Core
{
    public sealed class SerializationManager
    {
        #region Constructors
        private SerializationManager()
        { 
            //no imple
        }
        #endregion

        #region Public Members
        public static T XmlDeserialize<T>(XmlDocument xmlDocument)
        {
            XmlNodeReader _xmlNodeReader = new XmlNodeReader(xmlDocument);

            XmlSerializerFactory _xFactory = new XmlSerializerFactory();
            XmlSerializer _serializer = _xFactory.CreateSerializer(typeof(T));

            T _object = (T)_serializer.Deserialize(_xmlNodeReader);

            return _object;
        }

        public static string XmlSerialize<T>(T objectInstance)
        {
            XmlSerializerFactory _xFactory = new XmlSerializerFactory();
            XmlSerializer _serializer = _xFactory.CreateSerializer(typeof(T));
            string _result = string.Empty;

            using (MemoryStream _stream = new MemoryStream())
            {

                _serializer.Serialize(_stream, objectInstance);

                _result = GetStringFromStream(_stream);
            }
            return _result;

        }
        #endregion

        #region Private Members

        private static string GetStringFromStream(MemoryStream memoryStream)
        {
            memoryStream.Position = 0;
            byte[] _byteArray = memoryStream.ToArray();
            Encoding _encoder = Encoding.UTF8;
            return _encoder.GetString(_byteArray);

        }
        #endregion

    }
}
